@@ -20,7 +20,11 @@ class Options:
     PER_MODULE_OPTIONS = {
         "ignore_missing_imports",
         "follow_imports",
-        "disallow_any",
+        "disallow_any_generics",
+        "disallow_any_unimported",
+        "disallow_any_expr",
+        "disallow_any_decorated",
+        "disallow_any_explicit",
         "disallow_subclassing_any",
         "disallow_untyped_calls",
         "disallow_untyped_defs",
@@ -41,6 +45,9 @@ class Options:
                                - {"debug_cache"})
 
     def __init__(self) -> None:
+        # Cache for clone_for_module()
+        self.clone_cache = {}  # type: Dict[str, Options]
+
         # -- build options --
         self.build_type = BuildType.STANDARD
         self.python_version = defaults.PYTHON3_VERSION
@@ -51,7 +58,13 @@ class Options:
         self.report_dirs = {}  # type: Dict[str, str]
         self.ignore_missing_imports = False
         self.follow_imports = 'normal'  # normal|silent|skip|error
-        self.disallow_any = []  # type: List[str]
+
+        # disallow_any options
+        self.disallow_any_generics = False
+        self.disallow_any_unimported = False
+        self.disallow_any_expr = False
+        self.disallow_any_decorated = False
+        self.disallow_any_explicit = False
 
         # Disallow calling untyped functions from typed ones
         self.disallow_untyped_calls = False
@@ -155,6 +168,7 @@ class Options:
         self.shadow_file = None  # type: Optional[Tuple[str, str]]
         self.show_column_numbers = False  # type: bool
         self.dump_graph = False
+        self.dump_deps = False
 
     def __eq__(self, other: object) -> bool:
         return self.__class__ == other.__class__ and self.__dict__ == other.__dict__
@@ -166,6 +180,14 @@ class Options:
         return 'Options({})'.format(pprint.pformat(self.__dict__))
 
     def clone_for_module(self, module: str) -> 'Options':
+        """Create an Options object that incorporates per-module options.
+
+        NOTE: Once this method is called all Options objects should be
+        considered read-only, else the caching might be incorrect.
+        """
+        res = self.clone_cache.get(module)
+        if res is not None:
+            return res
         updates = {}
         for pattern in self.per_module_options:
             if self.module_matches_pattern(module, pattern):
@@ -173,10 +195,12 @@ class Options:
                     del self.unused_configs[pattern]
                 updates.update(self.per_module_options[pattern])
         if not updates:
+            self.clone_cache[module] = self
             return self
         new_options = Options()
         new_options.__dict__.update(self.__dict__)
         new_options.__dict__.update(updates)
+        self.clone_cache[module] = new_options
         return new_options
 
     def module_matches_pattern(self, module: str, pattern: Pattern[str]) -> bool:
