@@ -8,16 +8,16 @@ from mypy import build
 from mypy.build import BuildManager, BuildSource, State
 from mypy.errors import Errors, CompileError
 from mypy.nodes import (
-    Node, MypyFile, SymbolTable, SymbolTableNode, TypeInfo, Expression
+    Node, MypyFile, SymbolTable, SymbolTableNode, TypeInfo, Expression, UNBOUND_IMPORTED
 )
 from mypy.options import Options
 from mypy.server.astmerge import merge_asts
 from mypy.server.subexpr import get_subexpressions
-from mypy.server.update import build_incremental_step, replace_modules_with_new_variants
+from mypy.server.update import FineGrainedBuildManager
 from mypy.strconv import StrConv, indent
 from mypy.test.config import test_temp_dir, test_data_prefix
 from mypy.test.data import parse_test_cases, DataDrivenTestCase, DataSuite
-from mypy.test.helpers import assert_string_arrays_equal
+from mypy.test.helpers import assert_string_arrays_equal, normalize_error_messages
 from mypy.test.testtypegen import ignore_node
 from mypy.types import TypeStrVisitor, Type
 from mypy.util import short_type, IdMapper
@@ -35,6 +35,9 @@ TYPES = 'TYPES'
 AST = 'AST'
 
 
+NOT_DUMPED_MODULES = ('builtins', 'typing', 'abc')
+
+
 class ASTMergeSuite(DataSuite):
     def __init__(self, *, update_data: bool) -> None:
         super().__init__(update_data=update_data)
@@ -67,33 +70,29 @@ class ASTMergeSuite(DataSuite):
         main_src = '\n'.join(testcase.input)
         messages, manager, graph = self.build(main_src)
         assert manager is not None, 'cases where CompileError occurred should not be run'
+        fine_grained_manager = FineGrainedBuildManager(manager, graph)
 
         a = []
         if messages:
             a.extend(messages)
 
-        shutil.copy(os.path.join(test_temp_dir, 'target.py.next'),
-                    os.path.join(test_temp_dir, 'target.py'))
+        target_path = os.path.join(test_temp_dir, 'target.py')
+        shutil.copy(os.path.join(test_temp_dir, 'target.py.next'), target_path)
 
-        a.extend(self.dump(manager.modules, graph, kind))
-
-        old_modules = dict(manager.modules)
-        old_subexpr = get_subexpressions(old_modules['target'])
-
-        new_file, new_types = self.build_increment(manager, 'target')
-        replace_modules_with_new_variants(manager,
-                                          graph,
-                                          old_modules,
-                                          {'target': new_file},
-                                          {'target': new_types})
+        a.extend(self.dump(manager, kind))
+        old_subexpr = get_subexpressions(manager.modules['target'])
 
         a.append('==>')
-        a.extend(self.dump(manager.modules, graph, kind))
+
+        new_file, new_types = self.build_increment(fine_grained_manager, 'target', target_path)
+        a.extend(self.dump(manager, kind))
 
         for expr in old_subexpr:
             # Verify that old AST nodes are removed from the expression type map.
             assert expr not in new_types
 
+        a = normalize_error_messages(a)
+
         assert_string_arrays_equal(
             testcase.output, a,
             'Invalid output ({}, line {})'.format(testcase.file,
@@ -101,11 +100,14 @@ class ASTMergeSuite(DataSuite):
 
     def build(self, source: str) -> Tuple[List[str], Optional[BuildManager], Dict[str, State]]:
         options = Options()
+        options.incremental = True
         options.use_builtins_fixtures = True
         options.show_traceback = True
-        options.cache_dir = os.devnull
+        main_path = os.path.join(test_temp_dir, 'main')
+        with open(main_path, 'w') as f:
+            f.write(source)
         try:
-            result = build.build(sources=[BuildSource('main', None, source)],
+            result = build.build(sources=[BuildSource(main_path, None, None)],
                                  options=options,
                                  alt_lib_path=test_temp_dir)
         except CompileError as e:
@@ -113,16 +115,18 @@ class ASTMergeSuite(DataSuite):
             return e.messages, None, {}
         return result.errors, result.manager, result.graph
 
-    def build_increment(self, manager: BuildManager,
-                        module_id: str) -> Tuple[MypyFile,
-                                                 Dict[Expression, Type]]:
-        module_dict, type_maps = build_incremental_step(manager, [module_id])
-        return module_dict[module_id], type_maps[module_id]
+    def build_increment(self, manager: FineGrainedBuildManager,
+                        module_id: str, path: str) -> Tuple[MypyFile,
+                                                            Dict[Expression, Type]]:
+        manager.update([(module_id, path)])
+        module = manager.manager.modules[module_id]
+        type_map = manager.type_maps[module_id]
+        return module, type_map
 
     def dump(self,
-             modules: Dict[str, MypyFile],
-             graph: Dict[str, State],
+             manager: BuildManager,
              kind: str) -> List[str]:
+        modules = manager.modules
         if kind == AST:
             return self.dump_asts(modules)
         elif kind == TYPEINFO:
@@ -130,14 +134,14 @@ class ASTMergeSuite(DataSuite):
         elif kind == SYMTABLE:
             return self.dump_symbol_tables(modules)
         elif kind == TYPES:
-            return self.dump_types(graph)
+            return self.dump_types(manager)
         assert False, 'Invalid kind %s' % kind
 
     def dump_asts(self, modules: Dict[str, MypyFile]) -> List[str]:
         a = []
         for m in sorted(modules):
-            if m == 'builtins':
-                # We don't support incremental checking of changes to builtins.
+            if m in NOT_DUMPED_MODULES:
+                # We don't support incremental checking of changes to builtins, etc.
                 continue
             s = modules[m].accept(self.str_conv)
             a.extend(s.splitlines())
@@ -146,8 +150,8 @@ class ASTMergeSuite(DataSuite):
     def dump_symbol_tables(self, modules: Dict[str, MypyFile]) -> List[str]:
         a = []
         for id in sorted(modules):
-            if id == 'builtins':
-                # We don't support incremental checking of changes to builtins.
+            if not is_dumped_module(id):
+                # We don't support incremental checking of changes to builtins, etc.
                 continue
             a.extend(self.dump_symbol_table(id, modules[id].names))
         return a
@@ -161,18 +165,24 @@ class ASTMergeSuite(DataSuite):
         return a
 
     def format_symbol_table_node(self, node: SymbolTableNode) -> str:
-        if node is None:
+        if node.node is None:
+            if node.kind == UNBOUND_IMPORTED:
+                return 'UNBOUND_IMPORTED'
             return 'None'
         if isinstance(node.node, Node):
-            return '{}<{}>'.format(str(type(node.node).__name__),
-                                   self.id_mapper.id(node.node))
-        # TODO: type_override?
-        return '?'
+            s = '{}<{}>'.format(str(type(node.node).__name__),
+                                self.id_mapper.id(node.node))
+        else:
+            s = '? ({})'.format(type(node.node))
+        if node.type_override:
+            override = self.format_type(node.type_override)
+            s += '(type_override={})'.format(override)
+        return s
 
     def dump_typeinfos(self, modules: Dict[str, MypyFile]) -> List[str]:
         a = []
         for id in sorted(modules):
-            if id == 'builtins':
+            if not is_dumped_module(id):
                 continue
             a.extend(self.dump_typeinfos_recursive(modules[id].names))
         return a
@@ -190,14 +200,14 @@ class ASTMergeSuite(DataSuite):
                       type_str_conv=self.type_str_conv)
         return s.splitlines()
 
-    def dump_types(self, graph: Dict[str, State]) -> List[str]:
+    def dump_types(self, manager: BuildManager) -> List[str]:
         a = []
         # To make the results repeatable, we try to generate unique and
         # deterministic sort keys.
-        for module_id in sorted(graph):
-            if module_id == 'builtins':
+        for module_id in sorted(manager.modules):
+            if not is_dumped_module(module_id):
                 continue
-            type_map = graph[module_id].type_checker.type_map
+            type_map = manager.saved_cache[module_id][2]
             if type_map:
                 a.append('## {}'.format(module_id))
                 for expr in sorted(type_map, key=lambda n: (n.line, short_type(n),
@@ -205,5 +215,12 @@ class ASTMergeSuite(DataSuite):
                     typ = type_map[expr]
                     a.append('{}:{}: {}'.format(short_type(expr),
                                                 expr.line,
-                                                typ.accept(self.type_str_conv)))
+                                                self.format_type(typ)))
         return a
+
+    def format_type(self, typ: Type) -> str:
+        return typ.accept(self.type_str_conv)
+
+
+def is_dumped_module(id: str) -> bool:
+    return id not in NOT_DUMPED_MODULES and (not id.startswith('_') or id == '__main__')
