@@ -7,25 +7,29 @@ from typing import List
 from mypy import defaults
 from mypy.myunit import Suite, AssertionFailure
 from mypy.test.helpers import assert_string_arrays_equal
-from mypy.test.data import parse_test_cases, DataDrivenTestCase
+from mypy.test.data import parse_test_cases, DataDrivenTestCase, DataSuite
 from mypy.test import config
 from mypy.parse import parse
 from mypy.errors import CompileError
 from mypy.options import Options
 
 
-class ParserSuite(Suite):
+class ParserSuite(DataSuite):
     parse_files = ['parse.test',
                    'parse-python2.test']
 
-    def cases(self) -> List[DataDrivenTestCase]:
+    @classmethod
+    def cases(cls) -> List[DataDrivenTestCase]:
         # The test case descriptions are stored in data files.
         c = []  # type: List[DataDrivenTestCase]
-        for f in self.parse_files:
+        for f in cls.parse_files:
             c += parse_test_cases(
                 os.path.join(config.test_data_prefix, f), test_parser)
         return c
 
+    def run_case(self, testcase: DataDrivenTestCase) -> None:
+        test_parser(testcase)
+
 
 def test_parser(testcase: DataDrivenTestCase) -> None:
     """Perform a single parser test case.
@@ -42,6 +46,7 @@ def test_parser(testcase: DataDrivenTestCase) -> None:
     try:
         n = parse(bytes('\n'.join(testcase.input), 'ascii'),
                   fnam='main',
+                  module='__main__',
                   errors=None,
                   options=options)
         a = str(n).split('\n')
@@ -57,20 +62,26 @@ def test_parser(testcase: DataDrivenTestCase) -> None:
 INPUT_FILE_NAME = 'file'
 
 
-class ParseErrorSuite(Suite):
-    def cases(self) -> List[DataDrivenTestCase]:
+class ParseErrorSuite(DataSuite):
+    @classmethod
+    def cases(cls) -> List[DataDrivenTestCase]:
         # Test case descriptions are in an external file.
         return parse_test_cases(os.path.join(config.test_data_prefix,
                                              'parse-errors.test'),
                                 test_parse_error)
 
+    def run_case(self, testcase: DataDrivenTestCase) -> None:
+        test_parse_error(testcase)
+
 
 def test_parse_error(testcase: DataDrivenTestCase) -> None:
     try:
         # Compile temporary file. The test file contains non-ASCII characters.
-        parse(bytes('\n'.join(testcase.input), 'utf-8'), INPUT_FILE_NAME, None, Options())
+        parse(bytes('\n'.join(testcase.input), 'utf-8'), INPUT_FILE_NAME, '__main__', None,
+              Options())
         raise AssertionFailure('No errors reported')
     except CompileError as e:
+        assert e.module_with_blocker == '__main__'
         # Verify that there was a compile error and that the error messages
         # are equivalent.
         assert_string_arrays_equal(
