@@ -113,7 +113,7 @@ class ExpressionVisitor(Generic[T]):
         pass
 
     @abstractmethod
-    def visit_func_expr(self, o: 'mypy.nodes.FuncExpr') -> T:
+    def visit_lambda_expr(self, o: 'mypy.nodes.LambdaExpr') -> T:
         pass
 
     @abstractmethod
@@ -156,6 +156,10 @@ class ExpressionVisitor(Generic[T]):
     def visit_namedtuple_expr(self, o: 'mypy.nodes.NamedTupleExpr') -> T:
         pass
 
+    @abstractmethod
+    def visit_enum_call_expr(self, o: 'mypy.nodes.EnumCallExpr') -> T:
+        pass
+
     @abstractmethod
     def visit_typeddict_expr(self, o: 'mypy.nodes.TypedDictExpr') -> T:
         pass
@@ -481,7 +485,7 @@ class NodeVisitor(Generic[T], ExpressionVisitor[T], StatementVisitor[T]):
     def visit_type_application(self, o: 'mypy.nodes.TypeApplication') -> T:
         pass
 
-    def visit_func_expr(self, o: 'mypy.nodes.FuncExpr') -> T:
+    def visit_lambda_expr(self, o: 'mypy.nodes.LambdaExpr') -> T:
         pass
 
     def visit_list_comprehension(self, o: 'mypy.nodes.ListComprehension') -> T:
@@ -514,6 +518,9 @@ class NodeVisitor(Generic[T], ExpressionVisitor[T], StatementVisitor[T]):
     def visit_namedtuple_expr(self, o: 'mypy.nodes.NamedTupleExpr') -> T:
         pass
 
+    def visit_enum_call_expr(self, o: 'mypy.nodes.EnumCallExpr') -> T:
+        pass
+
     def visit_typeddict_expr(self, o: 'mypy.nodes.TypedDictExpr') -> T:
         pass
 
